from openpyxl.reader.excel import ExcelReader
from openpyxl.packaging.relationship import (
    RelationshipList,
    get_dependents,
    get_rels_path,
    get_rel,
)
from openpyxl.xml.constants import (
    COMMENTS_NS,
)
from openpyxl.comments.comment_sheet import CommentSheet
from openpyxl.drawing.spreadsheet_drawing import SpreadsheetDrawing
from openpyxl.xml.functions import fromstring
from .read_only_worksheet import SynoReadOnlyWorksheet

from openpyxl.xml.constants import IMAGE_NS
from openpyxl.chart.chartspace import ChartSpace
from openpyxl.chart.reader import read_chart
from openpyxl.cell.text import Text
from .richtext import parse_richtext


class SynoExcelReader(ExcelReader):

    def __init__(self, filename):
        ExcelReader.__init__(self, filename, read_only=True, keep_vba=False)

    def read_worksheets(self):
        for sheet, rel in self.parser.find_sheets():
            if rel.target not in self.valid_files:
                continue

            # Treat chartsheets as normal sheets
            # if 'chartsheet' in rel.Type:
            #    self.read_chartsheet(sheet, rel)
            #    continue

            rels_path = get_rels_path(rel.target)
            rels = RelationshipList()
            if rels_path in self.valid_files:
                rels = get_dependents(self.archive, rels_path)

            ws = SynoReadOnlyWorksheet(
                self.wb, sheet.name, rel.target, self.shared_strings)
            ws._rels = rels
            self.wb._sheets.append(ws)

            # assign any comments to cells
            for r in rels.find(COMMENTS_NS):
                src = self.archive.read(r.target)
                comment_sheet = CommentSheet.from_tree(fromstring(src))
                for ref, comment in comment_sheet.comments:
                    if comment.text:
                        ws.set_comment(ref, comment.text)

            drawings = rels.find(SpreadsheetDrawing._rel_type)
            for rrel in drawings:
                charts, images = find_drawings(self.archive, rrel.target)
                for c in charts:
                    ws.charts.append(c)
                for im in images:
                    ws.images.append(im)

            ws.sheet_state = sheet.state

    def extract_file(self, name, dest):
        return self.archive.extract(name, dest)


    def read_theme(self):
        ExcelReader.read_theme(self)
        if self.wb.loaded_theme:
            self.wb.theme_colors = read_theme_colors(self.wb.loaded_theme)


    def read(self):
        ExcelReader.read(self)
        if self.shared_strings:
            for index, text in enumerate(self.shared_strings):
                if isinstance(text, Text):
                    self.shared_strings[index] = parse_richtext(text, self.wb)


def find_drawings(archive, path):
    src = archive.read(path)
    tree = fromstring(src)
    try:
        drawing = SpreadsheetDrawing.from_tree(tree)
    except TypeError:
        return [], []

    rels_path = get_rels_path(path)
    deps = []
    if rels_path in archive.namelist():
        deps = get_dependents(archive, rels_path)

    charts = []
    for rel in drawing._chart_rels:
        cs = get_rel(archive, deps, rel.id, ChartSpace)
        chart = read_chart(cs)
        chart.anchor = rel.anchor
        charts.append(chart)

    images = []
    for rel in drawing._blip_rels:
        dep = deps[rel.embed]
        if dep.Type == IMAGE_NS:
            images.append({'name': dep.target, 'anchor': rel.anchor})

    return charts, images


def read_theme_colors(xmlstring):
    color_list = []
    ns = {'a': 'http://schemas.openxmlformats.org/drawingml/2006/main'}
    root = fromstring(xmlstring)
    clrScheme = root.find('./a:themeElements/a:clrScheme', ns)
    if clrScheme is None:
        return color_list
    ns_repr = '{%s}' % ns['a']
    offset_map = {'lt1': 0, 'dk1': 1, 'lt2': 2, 'dk2': 3}
    offset = 4
    for child in clrScheme:
        tag_name = child.tag.replace(ns_repr, '', 1)
        color = None
        index = 0
        if tag_name in offset_map:
            index = offset_map[tag_name]
        else:
            index = offset
            offset += 1

        sysClr = child.find('a:sysClr', ns)
        if sysClr is not None:
            color = sysClr.get('lastClr')
        elif child.find('a:srgbClr', ns) is not None:
            color = child.find('a:srgbClr', ns).get('val')

        while index >= len(color_list):
            color_list.append(None)
        color_list[index] = color.lower() if color else None

    return color_list
