<?php
/**
 * This file is part of PHPPresentation - A pure PHP library for reading and writing
 * presentations documents.
 *
 * PHPPresentation is free software distributed under the terms of the GNU Lesser
 * General Public License version 3 as published by the Free Software Foundation.
 *
 * For the full copyright and license information, please read the LICENSE
 * file that was distributed with this source code. For the full list of
 * contributors, visit https://github.com/PHPOffice/PHPPresentation/contributors.
 *
 * @link        https://github.com/PHPOffice/PHPPresentation
 * @copyright   2009-2015 PHPPresentation contributors
 * @license     http://www.gnu.org/licenses/lgpl.txt LGPL version 3
 */

namespace PhpOffice\PhpPresentation\Shape\Chart\Type;

use PhpOffice\PhpPresentation\ComparableInterface;
use PhpOffice\PhpPresentation\Shape\Chart\Series;

/**
 * \PhpOffice\PhpPresentation\Shape\Chart\Type
 */
abstract class AbstractType implements ComparableInterface
{
    /** Orientation of bars */
    const DIRECTION_VERTICAL = 'col';
    const DIRECTION_HORIZONTAL = 'bar';

    /** Grouping of bars */
    const GROUPING_CLUSTERED = 'clustered'; //Chart series are drawn next to each other along the category axis.
    const GROUPING_STACKED = 'stacked'; //Chart series are drawn next to each other on the value axis.
    const GROUPING_PERCENTSTACKED = 'percentStacked'; //Chart series are drawn next to each other along the value axis and scaled to total 100%

    /**
     * Orientation of bars
     *
     * @var string
     */
    protected $direction = self::DIRECTION_VERTICAL;


    /**
     * Grouping of bars
     *
     * @var string
     */
    protected $grouping = self::GROUPING_CLUSTERED;

    /**
     * Has Axis X?
     *
     * @var boolean
     */
    protected $hasAxisX = true;

    /**
     * Has Axis Y?
     *
     * @var boolean
     */
    protected $hasAxisY = true;

    /**
     * Hash index
     *
     * @var string
     */
    private $hashIndex;
    
    /**
     * Data
     *
     * @var array
     */
    private $data = array();

    private $marker = false;

    /**
     * Set orientation
     *
     * @param string                          $value
     * @return \PhpOffice\PhpPresentation\Shape\Chart\Type\Bar
     */
    public function setDirection($value = self::DIRECTION_VERTICAL)
    {
        $this->direction = $value;
        return $this;
    }

    /**
     * Get orientation
     *
     * @return string
     */
    public function getDirection()
    {
        return $this->direction;
    }

    /**
     * Set bar grouping (stack or expanded style bar)
     *
     * @param string                          $value
     * @return \PhpOffice\PhpPresentation\Shape\Chart\Type\Bar
     */
    public function setGrouping($value = self::GROUPING_CLUSTERED)
    {
        $this->grouping = $value;
        return $this;
    }

    /**
     * Get grouping  (stack or expanded style bar)
     *
     * @return string
     */
    public function getGrouping()
    {
        return $this->grouping;
    }

    /**
     * Has Axis X?
     *
     * @return boolean
     */
    public function hasAxisX()
    {
        return $this->hasAxisX;
    }

    /**
     * Has Axis Y?
     *
     * @return boolean
     */
    public function hasAxisY()
    {
        return $this->hasAxisY;
    }

    /**
     * Get hash index
     *
     * Note that this index may vary during script execution! Only reliable moment is
     * while doing a write of a workbook and when changes are not allowed.
     *
     * @return string Hash index
     */
    public function getHashIndex()
    {
        return $this->hashIndex;
    }

    /**
     * Set hash index
     *
     * Note that this index may vary during script execution! Only reliable moment is
     * while doing a write of a workbook and when changes are not allowed.
     *
     * @param string $value Hash index
     */
    public function setHashIndex($value)
    {
        $this->hashIndex = $value;
        return $this;
    }

    /**
     * Add Series
     *
     * @param  \PhpOffice\PhpPresentation\Shape\Chart\Series $value
     * @return self
     */
    public function addSeries(Series $value)
    {
        $this->data[] = $value;
        return $this;
    }

    /**
     * Get Series
     *
     * @return \PhpOffice\PhpPresentation\Shape\Chart\Series[]
     */
    public function getSeries()
    {
        return $this->data;
    }

    /**
     * Set Series
     *
     * @param  array $value Array of \PhpOffice\PhpPresentation\Shape\Chart\Series
     * @return self
     */
    public function setSeries($value = array())
    {
        $this->data = $value;
        return $this;
    }

    /**
     * Get Data
     *
     * @deprecated getSeries
     */
    public function getData()
    {
        return $this->getSeries();
    }

    /**
     * Set Data
     *
     * @deprecated setSeries
     */
    public function setData($value = array())
    {
        return $this->setSeries($value);
    }

    public function isMarker()
    {
        return $this->marker;
    }

    public function setMarker()
    {
        $this->marker = true;
        return $this;
    }

    /**
     * @return mixed
     * @link http://php.net/manual/en/language.oop5.cloning.php
     */
    public function __clone()
    {
        $arrayClone = array();
        foreach ($this->data as $itemSeries) {
            $arrayClone[] = clone $itemSeries;
        }
        $this->data = $arrayClone;
    }
}
