<?php
/**
 * This file is part of PHPPresentation - A pure PHP library for reading and writing
 * presentations documents.
 *
 * PHPPresentation is free software distributed under the terms of the GNU Lesser
 * General Public License version 3 as published by the Free Software Foundation.
 *
 * For the full copyright and license information, please read the LICENSE
 * file that was distributed with this source code. For the full list of
 * contributors, visit https://github.com/PHPOffice/PHPPresentation/contributors.
 *
 * @link        https://github.com/PHPOffice/PHPPresentation
 * @copyright   2009-2015 PHPPresentation contributors
 * @license     http://www.gnu.org/licenses/lgpl.txt LGPL version 3
 */

namespace PhpOffice\PhpPresentation\Style;

use PhpOffice\PhpPresentation\ComparableInterface;

/**
 * \PhpOffice\PhpPresentation\Style\Font
 */
class Font implements ComparableInterface
{
    const STATE_NOTSET = 'notset';
    const STATE_ON = 'on';
    const STATE_OFF = 'off';

    /* Underline types */
    const UNDERLINE_NOTSET = 'notset';
    const UNDERLINE_NONE = 'none';
    const UNDERLINE_DASH = 'dash';
    const UNDERLINE_DASHHEAVY = 'dashHeavy';
    const UNDERLINE_DASHLONG = 'dashLong';
    const UNDERLINE_DASHLONGHEAVY = 'dashLongHeavy';
    const UNDERLINE_DOUBLE = 'dbl';
    const UNDERLINE_DOTHASH = 'dotDash';
    const UNDERLINE_DOTHASHHEAVY = 'dotDashHeavy';
    const UNDERLINE_DOTDOTDASH = 'dotDotDash';
    const UNDERLINE_DOTDOTDASHHEAVY = 'dotDotDashHeavy';
    const UNDERLINE_DOTTED = 'dotted';
    const UNDERLINE_DOTTEDHEAVY = 'dottedHeavy';
    const UNDERLINE_HEAVY = 'heavy';
    const UNDERLINE_SINGLE = 'sng';
    const UNDERLINE_WAVY = 'wavy';
    const UNDERLINE_WAVYDOUBLE = 'wavyDbl';
    const UNDERLINE_WAVYHEAVY = 'wavyHeavy';
    const UNDERLINE_WORDS = 'words';

    /**
     * Name
     *
     * @var string
     */
    private $name;

    /**
     * Font Size
     *
     * @var float|int
     */
    private $size;

    /**
     * Bold
     *
     * @var boolean
     */
    private $bold;

    /**
     * Italic
     *
     * @var boolean
     */
    private $italic;

    /**
     * Superscript
     *
     * @var boolean
     */
    private $superScript;

    /**
     * Subscript
     *
     * @var boolean
     */
    private $subScript;

    /**
     * Underline
     *
     * @var string
     */
    private $underline;

    /**
     * Strikethrough
     *
     * @var boolean
     */
    private $strikethrough;

    /**
     * Font color
     *
     * @var \PhpOffice\PhpPresentation\Style\Color
     */
    private $color;

    /**
     * Highlight
     *
     * @var \PhpOffice\PhpPresentation\Style\Color
     */
     private $highlight;

    /**
     * Character Spacing
     *
     * @var int
     */
    private $characterSpacing;

    private $idx;

    /**
     * Hash index
     *
     * @var string
     */
    private $hashIndex;

    private $baseline = 0;
    private $langFontMap = array();

    /**
     * Create a new \PhpOffice\PhpPresentation\Style\Font
     */
    public function __construct()
    {
        // Initialise values
        $this->name             = '';
        $this->size             = 0;
        $this->characterSpacing = 0;
        $this->bold             = self::STATE_NOTSET;
        $this->italic           = self::STATE_NOTSET;
        $this->superScript      = self::STATE_NOTSET;
        $this->subScript        = self::STATE_NOTSET;
        $this->underline        = self::UNDERLINE_NOTSET;
        $this->strikethrough    = self::STATE_NOTSET;
        $this->idx = '';
    }

    /**
     * Get Name
     *
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Set Name
     *
     * @param  string                   $pValue
     * @return \PhpOffice\PhpPresentation\Style\Font
     */
    public function setName($pValue = 'Calibri')
    {
        if ($pValue == '') {
            $pValue = 'Calibri';
        }
        $this->name = $pValue;

        return $this;
    }

    /**
     * Get Character Spacing
     *
     * @return double
     */
    public function getCharacterSpacing()
    {
        return $this->characterSpacing;
    }

    /**
     * Set Character Spacing
     * Value in pt
     * @param float|int $pValue
     * @return \PhpOffice\PhpPresentation\Style\Font
     */
    public function setCharacterSpacing($pValue = 0)
    {
        if ($pValue == '') {
            $pValue = 0;
        }
        $this->characterSpacing = $pValue * 100;

        return $this;
    }

    /**
     * Get Size
     *
     * @return double
     */
    public function getSize()
    {
        return $this->size;
    }

    /**
     * Set Size
     *
     * @param float|int $pValue
     * @return \PhpOffice\PhpPresentation\Style\Font
     */
    public function setSize($pValue = 10)
    {
        if ($pValue == '') {
            $pValue = 10;
        }
        $this->size = $pValue;

        return $this;
    }

    /**
     * Get Bold
     *
     */
    public function getBold()
    {
        return $this->bold;
    }

    /**
     * Set Bold
     *
     * @return \PhpOffice\PhpPresentation\Style\Font
     */
    public function setBold($pValue = self::STATE_NOTSET)
    {
        if (self::STATE_NOTSET !== $pValue && self::STATE_ON === $pValue && self::STATE_OFF === $pValue) {
            $pValue = self::STATE_NOTSET;
        }
        $this->bold = $pValue;

        return $this;
    }

    /**
     * Get Italic
     *
     */
    public function getItalic()
    {
        return $this->italic;
    }

    /**
     * Set Italic
     *
     * @return \PhpOffice\PhpPresentation\Style\Font
     */
    public function setItalic($pValue = self::STATE_NOTSET)
    {
        if (self::STATE_NOTSET !== $pValue && self::STATE_ON === $pValue && self::STATE_OFF === $pValue) {
            $pValue = self::STATE_NOTSET;
        }
        $this->italic = $pValue;

        return $this;
    }

    /**
     * Get SuperScript
     *
     */
    public function getSuperScript()
    {
        return $this->superScript;
    }

    /**
     * Set SuperScript
     *
     * @return \PhpOffice\PhpPresentation\Style\Font
     */
    public function setSuperScript($pValue = self::STATE_NOTSET)
    {
        if (self::STATE_ON === $pValue) {
            $this->subScript = self::STATE_NOTSET;
            $this->superScript = self::STATE_ON;
        } else {
            $this->superScript = self::STATE_NOTSET;
        }

        return $this;
    }

    /**
     * Get SubScript
     *
     */
    public function getSubScript()
    {
        return $this->subScript;
    }

    /**
     * Set SubScript
     *
     * @return \PhpOffice\PhpPresentation\Style\Font
     */
    public function setSubScript($pValue = self::STATE_NOTSET)
    {
        if (self::STATE_ON === $pValue) {
            $this->superScript = self::STATE_NOTSET;
            $this->subScript = self::STATE_ON;
        } else {
            $this->subScript = self::STATE_NOTSET;
        }

        return $this;
    }

    /**
     * Get Underline
     *
     * @return string
     */
    public function getUnderline()
    {
        return $this->underline;
    }

    /**
     * Set Underline
     *
     * @param  string                   $pValue \PhpOffice\PhpPresentation\Style\Font underline type
     * @return \PhpOffice\PhpPresentation\Style\Font
     */
    public function setUnderline($pValue = self::UNDERLINE_NOTSET)
    {
        if ($pValue == '') {
            $pValue = self::UNDERLINE_NOTSET;
        }
        $this->underline = $pValue;

        return $this;
    }

    /**
     * Get Strikethrough
     *
     */
    public function getStrikethrough()
    {
        return $this->strikethrough;
    }

    /**
     * Set Strikethrough
     *
     * @return \PhpOffice\PhpPresentation\Style\Font
     */
    public function setStrikethrough($pValue = self::STATE_NOTSET)
    {
        if (self::STATE_NOTSET !== $pValue && self::STATE_ON === $pValue && self::STATE_OFF === $pValue) {
            $pValue = self::STATE_NOTSET;
        }
        $this->strikethrough = $pValue;

        return $this;
    }

    /**
     * Get Color
     *
     * @return \PhpOffice\PhpPresentation\Style\Color|\PhpOffice\PhpPresentation\Style\SchemeColor
     */
    public function getColor()
    {
        return $this->color;
    }

    /**
     * Set Color
     *
     * @param  \PhpOffice\PhpPresentation\Style\Color|\PhpOffice\PhpPresentation\Style\SchemeColor $pValue
     * @throws \Exception
     * @return \PhpOffice\PhpPresentation\Style\Font
     */
    public function setColor($pValue = null)
    {
        if (!$pValue instanceof Color) {
            throw new \Exception('$pValue must be an instance of \PhpOffice\PhpPresentation\Style\Color');
        }
        $this->color = $pValue;

        return $this;
    }

    /**
     * Get Highlight
     *
     * @return \PhpOffice\PhpPresentation\Style\Color|\PhpOffice\PhpPresentation\Style\Color
     */
    public function getHighlight()
    {
        return $this->highlight;
    }

    /**
     * Set Highlight
     *
     * @param  \PhpOffice\PhpPresentation\Style\Color|\PhpOffice\PhpPresentation\Style\Color $pValue
     * @throws \Exception
     * @return \PhpOffice\PhpPresentation\Style\Font
     */
    public function setHighlight($pValue = null)
    {
        if (!$pValue instanceof Color) {
            throw new \Exception('$pValue must be an instance of \PhpOffice\PhpPresentation\Style\Color');
        }
        $this->highlight = $pValue;

        return $this;
    }

    /**
     * Get hash code
     *
     * @return string Hash code
     */
    public function getHashCode()
    {
        $color = (!is_null($this->color)) ? $this->color->getHashCode() : '';
        return md5($this->name . $this->size . $this->characterSpacing . ($this->bold ? 't' : 'f') . ($this->italic ? 't' : 'f') . ($this->superScript ? 't' : 'f') . ($this->subScript ? 't' : 'f') . $this->underline . ($this->strikethrough ? 't' : 'f') . $color . __CLASS__);
    }

    /**
     * Get hash index
     *
     * Note that this index may vary during script execution! Only reliable moment is
     * while doing a write of a workbook and when changes are not allowed.
     *
     * @return string Hash index
     */
    public function getHashIndex()
    {
        return $this->hashIndex;
    }

    /**
     * Set hash index
     *
     * Note that this index may vary during script execution! Only reliable moment is
     * while doing a write of a workbook and when changes are not allowed.
     *
     * @param string $value Hash index
     */
    public function setHashIndex($value)
    {
        $this->hashIndex = $value;
    }

    public function getBaseline()
    {
        return $this->baseline;
    }

    public function setBaseline($baseline)
    {
        $this->baseline = floatval($baseline);
        return $this;
    }

    public function getIdx()
    {
        return $this->idx;
    }

    public function setIdx($value)
    {
        $this->idx = $value;
        return $this;
    }

    public function getLangFont($lang = '')
    {
        $typeface = $this->langFontMap[$lang];
        if (isset($typeface)) {
            return $typeface;
        }
        return 'Calibri';
    }

    public function setLangFont($lang, $typeface = 'Calibri')
    {
        if (isset($lang) && !empty($lang)) {
            $this->langFontMap[$lang] = $typeface;
        }
        return $this;
    }

    public function getLangFontMap()
    {
        return $this->langFontMap;
    }
}
