<?php
/**
 * This file is part of PHPWord - A pure PHP library for reading and writing
 * word processing documents.
 *
 * PHPWord is free software distributed under the terms of the GNU Lesser
 * General Public License version 3 as published by the Free Software Foundation.
 *
 * For the full copyright and license information, please read the LICENSE
 * file that was distributed with this source code. For the full list of
 * contributors, visit https://github.com/PHPOffice/PHPWord/contributors.
 *
 * @link        https://github.com/PHPOffice/PHPWord
 * @copyright   2010-2014 PHPWord contributors
 * @license     http://www.gnu.org/licenses/lgpl.txt LGPL version 3
 */

namespace PhpOffice\PhpWord\Writer\HTML_ProseMirror\Element;

/**
 * Table element HTML writer
 *
 * @since 0.10.0
 */
class Table extends AbstractElement
{
    /**
     * Write table
     *
     * @return string
     */
    public function write()
    {
        if (!$this->element instanceof \PhpOffice\PhpWord\Element\Table) {
            return '';
        }
        $tblStyle = $this->element->getStyle()->getFinalStyle();

        // Build table grid
        $grid = array();
        $rows = $this->element->getRows();
        for ($i = 0, $numRows = count($rows); $i < $numRows; ++$i) {
            $index = 0;
            $row = $rows[$i];
            $grid[] = array(
                'element' => $row,
                'height' => $row->getHeight(),
                'cells' => array());
            $cells = $row->getCells();
            for ($j = 0, $numCells = count($cells); $j < $numCells; ++$j) {
                $cell = $cells[$j];
                $cellStyle = $cell->getStyle();

                $bgColor = $cellStyle->getBgColor();

                if ($cellStyle->getVMerge() === 'continue') {
                    $k = $i - 1;
                    while (($k >= 0) && !$grid[$k]['cells'][$index]['real']) {
                        --$k;
                    }
                    if ($k < 0) {
                        // TODO
                        return '';
                    }
                    for ($l = 0, $colspan = $grid[$k]['cells'][$index]['colspan']; $l < $colspan; ++$l) {
                        ++$grid[$k]['cells'][$index]['rowspan'];
                        $grid[$i]['cells'][$index] = array(
                            'element' => $cell,
                            'width' => $cell->getWidth(),
                            'bgColor' => $bgColor,
                            'rowspan' => $grid[$k]['cells'][$index]['rowspan'],
                            'colspan' => $grid[$k]['cells'][$index]['colspan'],
                            'real' => false);
                        ++$index;
                    }
                    continue;
                }

                $colspan = $cellStyle->getGridSpan();
                if (!$colspan) {
                    $colspan = 1;
                }
                $colspan = (int)$colspan;
                for ($k = 0; $k < $colspan; ++$k) {
                    $grid[$i]['cells'][$index] = array(
                        'element' => $cell,
                        'width' => $cell->getWidth(),
                        'bgColor' => $bgColor,
                        'rowspan' => 1,
                        'colspan' => $colspan,
                        'real' => ($k === 0),
                        'borderLeftColor' => $cellStyle->getBorderLeftColor(),
                        'borderLeftStyle' => $cellStyle->getBorderLeftStyle(),
                        'noBorderLeft' => $cellStyle->checkNoBorder($cellStyle->getBorderLeftStyle()),
                        'borderLeftWidth' => $cellStyle->getBorderLeftSize(true),
                        'borderTopColor' => $cellStyle->getBorderTopColor(),
                        'borderTopStyle' => $cellStyle->getBorderTopStyle(),
                        'noBorderTop' => $cellStyle->checkNoBorder($cellStyle->getBorderTopStyle()),
                        'borderTopWidth' => $cellStyle->getBorderTopSize(true),
                        'borderRightColor' => $cellStyle->getBorderRightColor(),
                        'borderRightStyle' => $cellStyle->getBorderRightStyle(),
                        'noBorderRight' => $cellStyle->checkNoBorder($cellStyle->getBorderRightStyle()),
                        'borderRightWidth' => $cellStyle->getBorderRightSize(true),
                        'borderBottomColor' => $cellStyle->getBorderBottomColor(),
                        'borderBottomStyle' => $cellStyle->getBorderBottomStyle(),
                        'noBorderBottom' => $cellStyle->checkNoBorder($cellStyle->getBorderBottomStyle()),
                        'borderBottomWidth' => $cellStyle->getBorderBottomSize(true)
                    );
                    ++$index;
                }

            }
        }

        if (count($grid) < 1) {
            return '';
        }

        // Write grid
        $content = '<table';
        $align = $tblStyle->getAlign();
        switch ($align) {
            case 'right': {
                $content .= ' style="margin-left:auto;"';
                break;
            }
            case 'center': {
                $content .= ' style="margin-left:auto;margin-right:auto;"';
                break;
            }
            case 'left':
                // fall through
            default: {
                break;
            }
        }
        $content .= '>' . PHP_EOL; // <table>

        $colgroup = $this->element->getColgroup();
        if ($colgroup) {
            $width = 0;
            $colgroupHtml = '<colgroup>' . PHP_EOL;
            foreach ($colgroup as $col) {
                $w = (int)((int)$col['width'] / 15);
                $width += $w;
                $colgroupHtml .= '<col style="width:' . $w . 'px;">' . PHP_EOL;
            }
            $colgroupHtml .= '</colgroup>';
            $content .= $colgroupHtml;
        }

        $content .= '<tbody>' . PHP_EOL;
        for ($i = 0, $numRows = count($grid); $i < $numRows; ++$i) {
            $row = $grid[$i];
            $content .= '<tr';
            if ($row['height']) {
                $height = (int)$row['height'] / 20;
                $content .= " style=\"height:{$height}px;\"";
            }
            $revision = $row['element']->getRevision();
            if (is_array($revision)) {
                $id = $revision['id'];
                $op = array('del' => 'deletion', 'ins' => 'insertion')[$revision['op']];
                $content .= " data-syno-comment=\"{$id}\" data-syno-suggestion=\"{$op}\"";
            }
            $content .= '>' . PHP_EOL;
            for ($j = 0, $numCells = count($row['cells']); $j < $numCells; ++$j) {
                $cell = $row['cells'][$j];
                if (!$cell['real']) {
                    continue;
                }
                $writer = new Container($this->parentWriter, $cell['element']);
                $cellTag = $tblHeader ? 'th' : 'td';
                $content .= "<{$cellTag} rowspan=\"{$cell['rowspan']}\" colspan=\"{$cell['colspan']}\"";
                $style_str = "";
                if ($cell['width']) {
                    $width = (int)$cell['width'] / 20;
                    $style_str .= "width:{$width}px;";
                }
                if (!is_null($cell['bgColor'])) {
                    $color = $cell['bgColor'];
                    $style_str .= "background-color:#{$color};";
                }
                $sides = array('Top', 'Left', 'Bottom', 'Right');
                $sideColors = array();
                $sideWidths = array();

                // check boundaries
                if ( $i == 0 ) {
                    $sideColors['Top'] = $tblStyle->getBorderTopColor();
                    $sideWidths['Top'] = $tblStyle->getBorderTopSize(true);
                } else {
                    $sideColors['Top'] = $tblStyle->getBorderInsideHColor();
                    $sideWidths['Top'] = $tblStyle->getBorderInsideHSize(true);
                }
                if ( $i == $numRows-1 ) {
                    $sideColors['Bottom'] = $tblStyle->getBorderBottomColor();
                    $sideWidths['Bottom'] = $tblStyle->getBorderBottomSize(true);
                } else {
                    $sideColors['Bottom'] = $tblStyle->getBorderInsideHColor();
                    $sideWidths['Bottom'] = $tblStyle->getBorderInsideHSize(true);
                }
                if ( $j == 0 ) {
                    $sideColors['Left'] = $tblStyle->getBorderLeftColor();
                    $sideWidths['Left'] = $tblStyle->getBorderLeftSize(true);
                } else {
                    $sideColors['Left'] = $tblStyle->getBorderInsideVColor();
                    $sideWidths['Left'] = $tblStyle->getBorderInsideVSize(true);
                }
                if ( $j == $numCells-1 ) {
                    $sideColors['Right'] = $tblStyle->getBorderRightColor();
                    $sideWidths['Right'] = $tblStyle->getBorderRightSize(true);
                } else {
                    $sideColors['Right'] = $tblStyle->getBorderInsideVColor();
                    $sideWidths['Right'] = $tblStyle->getBorderInsideVSize(true);
                }
                foreach ( $sides as $side ) {
                    if ( empty($sideColors[$side]) ) {
                        $sideColors[$side] = \PhpOffice\PhpWord\Style\Cell::DEFAULT_BORDER_COLOR;
                    }
                    if ( !is_null($cell["border{$side}Color"]) ) {
                        $sideColors[$side] = $cell["border{$side}Color"];
                    }
                    if ( true === $cell["noBorder{$side}"] ) {
                        $sideWidths[$side] = 0;
                    }
                    else if ( !is_null($cell["border{$side}Width"]) ) {
                        $sideWidths[$side] = $cell["border{$side}Width"];
                    }
                }
                $transformBorderStyle = function ($value) {
                    return 'solid'; // TODO
                };

                $style_str .= "border-left-color:#{$sideColors['Left']};";
                $style_str .= 'border-left-style:' . $transformBorderStyle($cell['borderLeftStyle']) . ';';
                $style_str .= 'border-left-width:' . (intval($sideWidths['Left']) / 6) . 'px;';
                $style_str .= "border-top-color:#{$sideColors['Top']};";
                $style_str .= 'border-top-style:' . $transformBorderStyle($cell['borderTopStyle']) . ';';
                $style_str .= 'border-top-width:' . (intval($sideWidths['Top']) / 6) . 'px;';
                $style_str .= "border-right-color:#{$sideColors['Right']};";
                $style_str .= 'border-right-style:' . $transformBorderStyle($cell['borderRightStyle']) . ';';
                $style_str .= 'border-right-width:' . (intval($sideWidths['Right']) / 6) . 'px;';
                $style_str .= "border-bottom-color:#{$sideColors['Bottom']};";
                $style_str .= 'border-bottom-style:' . $transformBorderStyle($cell['borderBottomStyle']) . ';';
                $style_str .= 'border-bottom-width:' . (intval($sideWidths['Bottom']) / 6) . 'px;';
                if (!empty($style_str)) {
                    $content .= " style=\"$style_str\"";
                }
                if (is_array($revision)) {
                    $content .= " data-syno-comment=\"{$id}\" data-syno-suggestion=\"{$op}\"";
                }
                $content .= '>' . PHP_EOL;
                $content .= $writer->write();
                $content .= "</{$cellTag}>" . PHP_EOL;
            }
            $content .= '</tr>' . PHP_EOL;
        }
        $content .= '</tbody>' . PHP_EOL;
        $content .= '</table>' . PHP_EOL;

        return $content;
    }
}
