BEGIN;

DROP VIEW IF EXISTS UDC_TEMPLATE_PRIVILEGED_COUNT;
DROP TABLE IF EXISTS db_template_version;
CREATE TABLE IF NOT EXISTS db_template_version (
	version NUMERIC UNIQUE NOT NULL
);

DROP TRIGGER IF EXISTS db_template_version_no_delete ON db_template_version;
CREATE OR REPLACE FUNCTION db_template_version_no_delete ()
RETURNS trigger
LANGUAGE plpgsql AS $f$
BEGIN
	RAISE EXCEPTION 'You may not delete the DB version!';
END; $f$;

CREATE TRIGGER db_template_version_no_delete
BEFORE DELETE ON db_template_version
FOR EACH ROW EXECUTE PROCEDURE db_template_version_no_delete();

INSERT INTO db_template_version(version) VALUES (3);

DROP INDEX IF EXISTS idx_template_owner, idx_template_system, idx_template_ctime, idx_template_mtime, idx_template_ntype, idx_template_sort_order;
DROP TABLE IF EXISTS template;
CREATE TABLE IF NOT EXISTS template (
	object_id TEXT UNIQUE NOT NULL,
	commit_msg JSON,
	title TEXT NOT NULL DEFAULT '',
	brief TEXT,
	acl JSON,
	owner BIGINT NOT NULL,
	system BOOLEAN NOT NULL DEFAULT 'f',
	creator BIGINT,
	version TEXT NOT NULL,
	publish_version TEXT NOT NULL DEFAULT '',
	ctime BIGINT,
	mtime BIGINT,
	thumb JSON,
	link_id TEXT NOT NULL,
	ntype BIGINT NOT NULL,
	sort_order BIGINT DEFAULT 0,
	extra_info JSON
);
CREATE INDEX idx_template_owner ON template(owner);
CREATE INDEX idx_template_system ON template(system);
CREATE INDEX idx_template_ctime ON template(ctime);
CREATE INDEX idx_template_mtime ON template(mtime);
CREATE INDEX idx_template_ntype ON template(ntype);
CREATE INDEX idx_template_sort_order ON template(sort_order);

DROP INDEX IF EXISTS idx_template_link_object_id;
DROP TABLE IF EXISTS template_link;
CREATE TABLE IF NOT EXISTS template_link (
	id TEXT UNIQUE NOT NULL,
	object_id TEXT NOT NULL,
	category TEXT NOT NULL,
	ntype BIGINT NOT NULL,
	CONSTRAINT template_link_mapper_ukey UNIQUE (id, object_id)
);
CREATE INDEX idx_template_link_object_id ON template_link(object_id);

DROP INDEX IF EXISTS idx_template_perm_user_object_id;
DROP TABLE IF EXISTS template_perm_user;
CREATE TABLE IF NOT EXISTS template_perm_user (
	object_id TEXT NOT NULL,
	uid BIGINT NOT NULL,
	perm INTEGER NOT NULL,
	inherit BOOLEAN NOT NULL,
	CONSTRAINT template_perm_user_mapper_ukey UNIQUE (uid, object_id)
);
CREATE INDEX idx_template_perm_user_object_id ON template_perm_user(object_id);

DROP INDEX IF EXISTS idx_template_perm_group_object_id;
DROP TABLE IF EXISTS template_perm_group;
CREATE TABLE IF NOT EXISTS template_perm_group (
	object_id TEXT NOT NULL,
	gid BIGINT NOT NULL,
	perm INTEGER NOT NULL,
	inherit BOOLEAN NOT NULL,
	CONSTRAINT template_perm_group_mapper_ukey UNIQUE (gid, object_id)
);
CREATE INDEX idx_template_perm_group_object_id ON template_perm_group(object_id);

DROP INDEX IF EXISTS idx_template_perm_app_object_id;
DROP TABLE IF EXISTS template_perm_app;
CREATE TABLE IF NOT EXISTS template_perm_app (
	object_id TEXT UNIQUE NOT NULL,
	perm INTEGER NOT NULL,
	inherit BOOLEAN NOT NULL
);
CREATE INDEX idx_template_perm_app_object_id ON template_perm_app(object_id);

DROP INDEX IF EXISTS idx_template_tag_object_object_id, idx_template_tag_object_tag_object_id;
DROP TABLE IF EXISTS template_tag_object;
CREATE TABLE IF NOT EXISTS template_tag_object (
	object_id TEXT NOT NULL,
	tag_object_id TEXT NOT NULL,
	CONSTRAINT template_tag_object_mapper_ukey UNIQUE (object_id, tag_object_id)
);
CREATE INDEX idx_template_tag_object_object_id ON template_tag_object(object_id);
CREATE INDEX idx_template_tag_object_tag_object_id ON template_tag_object(tag_object_id);

DROP INDEX IF EXISTS idx_template_tag_upper_title, idx_template_tag_position, idx_template_tag_system, idx_template_tag_deleted;
DROP TABLE IF EXISTS template_tag;
CREATE TABLE IF NOT EXISTS template_tag (
	object_id TEXT UNIQUE NOT NULL,
	title TEXT UNIQUE NOT NULL DEFAULT '',
	position BIGINT NOT NULL DEFAULT (extract(epoch from now()) * 1000),
	system BOOLEAN NOT NULL DEFAULT 'f',
	deleted BOOLEAN NOT NULL DEFAULT 'f'
);
CREATE INDEX idx_template_tag_upper_title ON template_tag(UPPER(title));
CREATE INDEX idx_template_tag_position ON template_tag(position);
CREATE INDEX idx_template_tag_system ON template_tag(system);
CREATE INDEX idx_template_tag_deleted ON template_tag(deleted);

DROP INDEX IF EXISTS idx_template_recent_atime, idx_template_recent_requester;
DROP TABLE IF EXISTS template_recent;
CREATE TABLE IF NOT EXISTS template_recent (
	object_id TEXT NOT NULL,
	requester BIGINT NOT NULL,
	atime BIGINT,
	CONSTRAINT template_recent_mapper_ukey UNIQUE (object_id, requester)
);
CREATE INDEX idx_template_recent_atime ON template_recent(atime);
CREATE INDEX idx_template_recent_requester ON template_recent(requester);

DROP TABLE IF EXISTS template_recent_vacuum;
CREATE TABLE IF NOT EXISTS template_recent_vacuum (
	requester BIGINT NOT NULL UNIQUE,
	times INTEGER NOT NULL DEFAULT 0
);

CREATE OR REPLACE FUNCTION insert_template_recent (in_object_id TEXT, in_requester BIGINT, in_atime BIGINT)
RETURNS BOOLEAN AS $$
	DECLARE
		result BOOLEAN;
	BEGIN
		INSERT INTO template_recent (object_id, requester, atime)
		SELECT in_object_id, in_requester, in_atime WHERE NOT EXISTS (SELECT 1 FROM template_recent WHERE object_id = in_object_id AND requester = in_requester)
		RETURNING TRUE INTO result;

		IF result IS NULL THEN
			result = FALSE;
		END IF;

		IF result = FALSE THEN
			UPDATE template_recent SET atime=in_atime WHERE object_id = in_object_id AND requester = in_requester RETURNING TRUE INTO result;
			INSERT INTO template_recent_vacuum (requester) SELECT in_requester WHERE NOT EXISTS (SELECT 1 FROM template_recent_vacuum WHERE requester = in_requester);
		END IF;

		return result;
	END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION vacuum_template_recent (in_uid BIGINT, in_gid BIGINT[])
RETURNS VOID AS $$
DECLARE
	result INTEGER;
BEGIN
	UPDATE template_recent_vacuum SET times = times + 1 WHERE requester=in_uid RETURNING times INTO result;

	IF result > 10 THEN
		DELETE FROM template_recent WHERE requester = in_uid AND template_recent.object_id NOT IN (
			SELECT object_id FROM template_perm_user WHERE uid = in_uid
			UNION SELECT object_id FROM template_perm_group WHERE gid = ANY (in_gid)
			UNION SELECT object_id FROM template_perm_app GROUP BY object_id
			UNION SELECT object_id FROM template WHERE system='t'
		);

		UPDATE template_recent_vacuum SET times = 0 WHERE requester = in_uid;
	END IF;

	RETURN;
END;
$$ LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION afterTemplateDeleted()
RETURNS "trigger" AS $$
	BEGIN
		DELETE FROM template_tag_object WHERE object_id=OLD.object_id;
		DELETE FROM template_perm_user WHERE object_id=OLD.object_id;
		DELETE FROM template_perm_group WHERE object_id=OLD.object_id;
		DELETE FROM template_perm_app WHERE object_id=OLD.object_id;
		DELETE FROM template_link WHERE object_id=OLD.object_id;
		DELETE FROM template_recent WHERE object_id=OLD.object_id;
		RETURN NULL;
	END;
$$ LANGUAGE plpgsql;

DROP TRIGGER IF EXISTS template_delete_trigger ON template;
CREATE TRIGGER "template_delete_trigger" AFTER DELETE
   ON template
   FOR EACH ROW
   EXECUTE PROCEDURE afterTemplateDeleted();

COMMIT;

